
/*
 * sym.h
 *
 * (C) Copyright 1995 Archie L. Cobbs
 */

/*
 * DEFINITIONS
 */

	OpaqueDecl(Symtab);

/*
 * FUNCTIONS
 */

	extern	Symtab		SymNew(Names db, unsigned itemsize);
	extern	void		*SymAdd(Symtab tab, unsigned id);
	extern	void		*SymFind(Symtab tab, unsigned id);
	extern	void		SymDump(Symtab tab, int (*Func)(void *item));

