
zp		=	13
abs		=	1234

; Check 65c02 stuff

start:
		ora	(zp)
		and	(zp)
		eor	(zp)
		xor	(zp)
		adc	(zp)
		sta	(zp)
		lda	(zp)
		cmp	(zp)
		sbc	(zp)

		dec
		dea
		inc
		ina

		bit	zp,x
		bit	abs,x
		bit	#>$ABACAB

		jmp	(zp, x)
		jmp	(abs, x)

		bra	start

		plx
		ply
		phx
		phy

		stz	zp
		stz	zp,x
		stz	abs
		stz	abs,x

		trb	zp
		trb	abs
		tsb	zp
		tsb	abs

; Check "negative space"

		lda	#1
		.space	-2
		.byte	$ac,$ac
		.space	-20

		.bss
		.align	4

		.space	10
temp2	=	*
		.space	-4
temp	=	*

