
/*
 * error.h
 *
 * (C) Copyright 1995 Archie L. Cobbs
 */

/*
 * DEFINITIONS
 */

/*
 * GLOBAL VARIABLES
 */

	extern	char			*gFileName;
	extern	int				gLineNumber;
	extern	int				gNumErrors;

/*
 * FUNCTIONS
 */

	extern	void			uerror(char *msg, ...);
	extern	volatile void	cerror(char *msg, ...);

