typedef union
	{
		unsigned		id;				/* Identifier */
		int				intval;			/* Integer literal */
		double			floatval;		/* Floating point literal */
		char			charval;		/* Character literal */
		struct TValue	textval;		/* String literal */
		struct IValue	ivalue;			/* Ivalue */
		struct List		ivlist;			/* List of ivalues */
		struct List		idlist;			/* List of identifiers */
		struct List		flist;			/* List of floats */
	} YYSTYPE;
#define	LOR	258
#define	LAND	259
#define	EQU	260
#define	NEQ	261
#define	LE	262
#define	GE	263
#define	LSH	264
#define	RSH	265
#define	LSB	266
#define	MSB	267
#define	ALIGN	268
#define	ASCII	269
#define	ASCIZ	270
#define	ASSERT	271
#define	BEGNN	272
#define	BSS	273
#define	BYTE	274
#define	CODE	275
#define	DATA	276
#define	DBYT	277
#define	DCI	278
#define	ELSE	279
#define	END	280
#define	ENDIF	281
#define	ENDM	282
#define	IF	283
#define	EXPAND	284
#define	FLOAT	285
#define	GLOBAL	286
#define	HIBIT	287
#define	MACRO	288
#define	LONG	289
#define	SPACE	290
#define	WORD	291
#define	ZERO	292
#define	IMPORT	293
#define	ZIMPORT	294
#define	INTLIT	295
#define	FLOATLIT	296
#define	CHARLIT	297
#define	TEXTLIT	298
#define	LABEL	299
#define	IDENT	300


extern YYSTYPE yylval;
