/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /home/daffy/u0/vern/flex/flex-2.4.7/RCS/flex.skl,v 1.2 94/08/03 11:13:24 vern Exp $
 */

#define FLEX_SCANNER

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#ifdef __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */


#ifdef __TURBOC__
#define YY_USE_CONST
#endif


#ifndef YY_USE_CONST
#ifndef const
#define const
#endif
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.
 */
#define YY_START ((yy_start - 1) / 2)

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file".  Now included
 * only for backward compatibility with previous versions of flex.
 */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#ifdef __cplusplus
extern "C" {
#endif
	extern int yywrap YY_PROTO(( void ));
#ifdef __cplusplus
	}
#endif

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

static void yyunput YY_PROTO(( int c, char *buf_ptr ));
void yyrestart YY_PROTO(( FILE *input_file ));
void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));

static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
static void yy_push_state YY_PROTO(( int new_state ));
static void yy_pop_state YY_PROTO(( void ));
static int yy_top_state YY_PROTO(( void ));

static void *yy_flex_alloc YY_PROTO(( unsigned int ));
static void *yy_flex_realloc YY_PROTO(( void *, unsigned int ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define INITIAL 0
#define Newline 1
#define Opcode 2
#define Args 3
#define Dquote 4
#define Squote 5
#define Ignore 6
#define Line1 7
#define Line2 8
typedef unsigned char YY_CHAR;
typedef int yy_state_type;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
extern char *yytext;
#define yytext_ptr yytext

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, const char *, int ));
#endif

#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( const char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = yy_cp - yy_bp; \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_END_OF_BUFFER 101
static const short int yy_accept[240] =
    {   0,
        0,    0,    0,    0,   16,   16,   53,   53,    0,    0,
        0,    0,   99,   99,    0,    0,    0,    0,  101,    1,
       10,    4,    2,    8,    5,    3,    6,   52,   16,   52,
        3,   18,   17,   78,   53,   65,   62,   67,   78,   65,
       64,   61,   64,   65,   56,   56,    3,   64,   78,   64,
       63,   66,   64,   98,   92,   79,   94,   80,   99,   14,
       15,   12,   14,   14,    4,    0,    0,    6,    7,   16,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   17,   53,   75,   68,   69,
       58,   54,   72,   59,   55,   56,    0,   70,   76,   74,

       77,   71,   63,   73,   93,   91,   90,   95,   89,   87,
       84,   88,   86,   81,   85,   82,   83,   97,   99,   12,
        0,    0,   13,    0,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   30,   51,   51,   51,   51,   51,
       51,   51,   48,   51,   51,   51,   51,   51,   51,   51,
       51,   59,   60,   57,   95,   96,    0,    0,   51,   51,
       51,   51,   24,   51,   51,   51,   51,   29,   51,   32,
       34,   51,   51,   51,   51,   51,   51,   43,   51,   51,
       51,   51,   51,   51,   60,   60,   95,   96,   11,    9,
       51,   51,   51,   51,   25,   26,   27,   28,   49,   51,

       33,   51,   51,   51,   51,   51,   51,   51,   44,   51,
       51,   45,   46,   51,   19,   20,   21,   51,   23,   50,
       51,   51,   51,   40,   51,   42,   51,   39,   31,   51,
       22,   35,   36,   37,   41,   38,   51,   47,    0
    } ;

static const int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        2,    2,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    7,    8,    9,   10,   11,
       12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
       21,   21,   21,   21,   21,   22,   22,   23,   24,   25,
       26,   27,   28,    1,   29,   29,   29,   29,   30,   29,
       31,   31,   31,   31,   31,   31,   31,   31,   31,   31,
       31,   31,   31,   31,   31,   31,   31,   31,   31,   31,
       32,   33,   34,   35,   36,    1,   37,   38,   39,   40,

       41,   42,   43,   44,   45,   31,   31,   46,   47,   48,
       49,   50,   51,   52,   53,   54,   55,   56,   57,   58,
       59,   60,    1,   61,    1,   62,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const int yy_meta[63] =
    {   0,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    3,    3,
        3,    3,    4,    1,    1,    1,    1,    1,    3,    3,
        5,    1,    1,    1,    1,    5,    3,    3,    3,    3,
        3,    3,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        1,    1
    } ;

static const short int yy_base[254] =
    {   0,
        0,    0,    0,    0,   62,    0,  124,    0,  184,  185,
      188,  189,  498,  497,  190,  194,  191,  197,  499,  502,
      502,  496,  502,  193,  502,  502,  474,  502,  494,  212,
      502,  502,    0,  502,  493,  468,  502,  176,    0,  185,
      484,  502,  502,  502,  256,  206,  502,  181,  466,  193,
        0,  502,  430,  502,  502,  502,  312,  502,    0,  502,
      502,  210,  445,  484,  486,  233,  442,  463,  502,  483,
        0,  191,  233,  435,  243,  237,  437,  436,  436,  242,
      431,  442,  428,  428,  193,    0,  474,  502,  502,  502,
        0,  226,  502,  278,  284,  290,    0,  502,  502,  502,

      502,  502,    0,  502,  502,  502,  502,  304,  502,  502,
      502,  502,  502,  502,  502,  502,  502,    0,    0,  307,
      427,  469,  468,  424,    0,  426,  194,  427,  416,  414,
      427,  412,  406,  419,    0,  410,  422,  406,  237,  411,
      410,  420,    0,  407,  402,  407,  415,  416,  400,  399,
      403,  316,  353,    0,  320,    0,  408,  407,  404,  401,
      404,  399,    0,  402,  401,  404,  386,    0,  398,  271,
        0,  293,  397,  400,  398,  390,  385,    0,  390,  380,
      392,  390,  380,  378,  357,  361,  342,    0,  502,  502,
      379,  298,  374,  376,    0,    0,    0,    0,    0,  377,

        0,  366,  359,  356,  317,  328,  305,  304,    0,  306,
      311,    0,    0,  302,    0,    0,    0,  294,    0,    0,
      307,  290,  286,    0,  275,    0,  266,    0,    0,  261,
        0,    0,    0,    0,    0,    0,  248,    0,  502,  383,
      388,  393,  398,  401,  404,  291,  407,  412,  417,  420,
      290,  221,  205
    } ;

static const short int yy_def[254] =
    {   0,
      240,  240,  239,    3,  239,    5,  239,    7,  241,  241,
      241,  241,  242,  242,  243,  243,  243,  243,  239,  239,
      239,  239,  239,  239,  239,  239,  244,  239,  239,  239,
      239,  239,  245,  239,  239,  239,  239,  239,  246,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      247,  239,  239,  239,  239,  239,  239,  239,  248,  239,
      239,  239,  239,  249,  239,  239,  239,  244,  239,  239,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  245,  239,  239,  239,  239,
      246,  239,  239,  239,  239,  239,  251,  239,  239,  239,

      239,  239,  247,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  252,  248,  239,
      239,  249,  249,  239,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  239,  239,  251,  239,  253,  239,  239,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  239,  239,  239,  253,  239,  239,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,

      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,  250,  250,
      250,  250,  250,  250,  250,  250,  250,  250,    0,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239
    } ;

static const short int yy_nxt[565] =
    {   0,
       21,   22,   23,   21,   21,   24,   21,   21,   21,   21,
       21,   21,   25,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   26,   21,   21,   21,   21,   27,   27,
       27,   21,   21,   21,   21,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       21,   21,   28,   29,   23,   28,   28,   28,   28,   28,
       28,   28,   28,   28,   28,   28,   28,   28,   30,   28,
       28,   28,   28,   28,   28,   31,   28,   32,   28,   28,
       33,   33,   33,   28,   28,   28,   28,   33,   33,   33,

       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
       33,   33,   28,   28,   34,   35,   23,   36,   37,   38,
       39,   40,   41,   42,   43,   43,   44,   44,   43,   44,
       34,   44,   45,   46,   46,   46,   43,   47,   48,   49,
       50,   43,   51,   51,   51,   52,   34,   52,   44,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   53,   44,   55,   55,   56,   56,
       55,   55,   61,   61,   66,   64,   61,   58,   58,   61,

       89,   64,   90,   92,   92,   98,   99,  188,   62,   62,
       62,   62,   62,   62,   62,   62,   57,   57,  101,  102,
       57,   57,   94,  156,   96,   96,   96,   96,  120,  120,
      120,  120,  160,  150,   66,   63,  126,  151,   67,   63,
       71,   71,   71,  127,   92,   92,  161,   71,   72,   73,
       74,   75,   76,   77,   78,   79,   80,   81,   82,   71,
       71,   71,   71,   71,   83,   71,   71,   71,   84,   71,
       71,   85,   94,  128,   95,   95,   95,   96,   67,  132,
      133,  134,  136,  143,  137,  129,  172,  138,  144,  145,
      173,  130,  154,   91,  139,  135,  152,  152,  152,  152,

       94,  238,   95,   95,   95,   96,   94,  153,   96,   96,
       96,   96,  237,   97,  105,  200,  106,  201,  153,  236,
      235,  107,  155,  155,  155,  120,  120,  120,  120,  202,
      108,  108,  108,  234,  152,  152,  152,  152,  187,  187,
      187,  203,  216,  233,  109,  153,  232,  231,  110,  111,
      230,  229,  112,  113,  228,  227,  153,  217,  226,  114,
      187,  187,  187,  115,  225,  116,  185,  117,  185,  118,
      224,  186,  186,  186,  186,  186,  186,  186,  186,  186,
      186,  186,  186,   20,   20,   20,   20,   20,   54,   54,
       54,   54,   54,   59,   59,   59,   59,   59,   60,   60,

       60,   60,   60,   68,   68,   68,   86,  223,   86,  103,
      222,  103,  119,  221,  119,  119,  119,  122,  220,  122,
      122,  122,  125,  219,  125,  218,  215,  214,  213,  212,
      211,  210,  209,  208,  207,  206,  205,  204,  199,  198,
      197,  196,  195,  194,  193,  192,  191,  190,  189,  184,
      183,  182,  181,  180,  179,  178,  177,  176,  175,  174,
      171,  170,  169,  168,  167,  166,  165,  164,  163,  162,
      159,  158,  123,  123,  157,   87,  149,  148,  147,  146,
      142,  141,  140,  131,   70,   69,  124,   65,  123,  121,
      104,  100,   93,   88,   87,   70,   69,   65,  239,   23,

       23,   19,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239
    } ;

static const short int yy_chk[565] =
    {   0,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,

        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    9,   10,    9,   10,
       11,   12,   15,   17,   24,   17,   16,   11,   12,   18,

       38,   18,   38,   40,   40,   48,   48,  253,   15,   15,
       15,   15,   16,   16,   16,   16,    9,   10,   50,   50,
       11,   12,   46,  252,   46,   46,   46,   46,   62,   62,
       62,   62,  127,   85,   66,   15,   72,   85,   24,   16,
       30,   30,   30,   72,   92,   92,  127,   30,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   45,   73,   45,   45,   45,   45,   66,   75,
       75,   75,   76,   80,   76,   73,  139,   76,   80,   80,
      139,   73,  251,  246,   76,   75,   94,   94,   94,   94,

       95,  237,   95,   95,   95,   95,   96,   94,   96,   96,
       96,   96,  230,   45,   57,  170,   57,  170,   94,  227,
      225,   57,  108,  108,  108,  120,  120,  120,  120,  172,
       57,   57,   57,  223,  152,  152,  152,  152,  155,  155,
      155,  172,  192,  222,   57,  152,  221,  218,   57,   57,
      214,  211,   57,   57,  210,  208,  152,  192,  207,   57,
      187,  187,  187,   57,  206,   57,  153,   57,  153,   57,
      205,  153,  153,  153,  153,  185,  185,  185,  185,  186,
      186,  186,  186,  240,  240,  240,  240,  240,  241,  241,
      241,  241,  241,  242,  242,  242,  242,  242,  243,  243,

      243,  243,  243,  244,  244,  244,  245,  204,  245,  247,
      203,  247,  248,  202,  248,  248,  248,  249,  200,  249,
      249,  249,  250,  194,  250,  193,  191,  184,  183,  182,
      181,  180,  179,  177,  176,  175,  174,  173,  169,  167,
      166,  165,  164,  162,  161,  160,  159,  158,  157,  151,
      150,  149,  148,  147,  146,  145,  144,  142,  141,  140,
      138,  137,  136,  134,  133,  132,  131,  130,  129,  128,
      126,  124,  123,  122,  121,   87,   84,   83,   82,   81,
       79,   78,   77,   74,   70,   68,   67,   65,   64,   63,
       53,   49,   41,   36,   35,   29,   27,   22,   19,   14,

       13,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  239,  239,  239,
      239,  239,  239,  239
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
char *yytext;
# line 1 "scan.y"
/*
 * scan.y
 *
 * (C) Copyright 1995 Archie L. Cobbs
 *
 */
/*
	The assembler makes two passes. The first pass is just to
	collect the defined symbols and their values (either known
	values or segment base relative offsets). So this all
	happens twice.
*/
/********************************************************************
 *						DEFINITIONS									*
 ********************************************************************/
/* Header stuff	*/
# line 23 "scan.y"

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include "util.h"
#include "error.h"
#include "names.h"
#include "asm.h"
#include "parse.tab.h"

/* Macros */

	#define	yyin			gInFile
	#define	YY_INPUT(b,r,m)	((r) = LoadBuf((b), (m)));

/* Return a specific pseudo-op */

	#define	Pseudo(x)		{												\
								BEGIN(Args);								\
								return(x);									\
							}

/* Add a character to current string literal */

	#define	add_char(c,h)	{												\
								if (textptr - textbuf >= MAX_STRING)		\
									uerror("string too long");				\
								else										\
									*textptr++ = (h) ? ((c) | gHiBit) : (c);\
							}

/* What to do when we get an unexpected character */

	#define	bad_char(s)		{												\
								if (!isprint(*yytext))						\
									uerror("illegal character: $%02X"		\
										" '%c'", *yytext, *yytext);			\
								else										\
									uerror(s);								\
								BEGIN(Ignore);								\
							}

/* Internal variables	*/

	static char				*textptr, textbuf[MAX_STRING];

/* Internal functions */

	static	int				LoadBuf(char *buf, int max);

/*
 * Start conditions, or "states":
 *
 * INITIAL	Before anything happens
 * Newline	We're at the very beginning of a line
 * Opcode	Looking for an opcode
 * Args		Looking for arguments to the opcode
 * Dquote	We're in the middle of a string
 * Squote	We're in the middle of a quoted character
 * Ignore	We're commenting till the end of the line
 * Line1,2	Reading a #line preprocessor directive
 */
/* Definitions	*/

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = getc( yyin ); \
		result = c == EOF ? 0 : 1; \
		buf[0] = (char) c; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

# line 112 "scan.y"


	/********************************************************************
	 *						RULES										*
	 ********************************************************************/

	/****** INITIAL MODE *******/


	if ( yy_init )
		{
#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( yy_current_buffer )
			yy_init_buffer( yy_current_buffer, yyin );
		else
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();

		yy_init = 0;
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 240 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 502 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_USER_ACTION
# line 120 "scan.y"
{
									unput(*yytext);
									BEGIN(Newline);
								}
	YY_BREAK
	/****** END OF LINE, END OF FILE, COMMENTS (CONSOLIDATED) ******/
case 2:
YY_USER_ACTION
# line 127 "scan.y"
{
									BEGIN(Newline);
									return('\n');
								}
	YY_BREAK
case YY_STATE_EOF(Opcode):
case YY_STATE_EOF(Args):
case YY_STATE_EOF(Ignore):
case YY_STATE_EOF(Line1):
case YY_STATE_EOF(Line2):
# line 132 "scan.y"
{
									BEGIN(Newline);
									return('\n');
								}
	YY_BREAK
case YY_STATE_EOF(Newline):
# line 137 "scan.y"
yyterminate();
	YY_BREAK
case 3:
YY_USER_ACTION
# line 139 "scan.y"
BEGIN(Ignore);
	YY_BREAK
	/****** NEWLINE MODE *******/
	/* Initial whitespace? Start looking for an opcode */
case 4:
YY_USER_ACTION
# line 145 "scan.y"
BEGIN(Opcode);
	YY_BREAK
	/* First character an asterisk? Ignore line */
case 5:
YY_USER_ACTION
# line 149 "scan.y"
BEGIN(Ignore);
	YY_BREAK
	/* Label? */
case 6:
YY_USER_ACTION
# line 153 "scan.y"
{
									yytext[yyleng] = 0;
									yylval.id = NameID(gNames, yytext);
									BEGIN(Opcode);
									return(LABEL);
								}
	YY_BREAK
case 7:
YY_USER_ACTION
# line 159 "scan.y"
{
									yytext[yyleng - 1] = 0;
									yylval.id = NameID(gNames, yytext);
									BEGIN(Opcode);
									return(LABEL);
								}
	YY_BREAK
	/* Line directive? */
case 8:
# line 169 "scan.y"
case 9:
YY_USER_ACTION
# line 169 "scan.y"
{
									BEGIN(Line1);
								}
	YY_BREAK
	/* Weird char? */
case 10:
YY_USER_ACTION
# line 175 "scan.y"
bad_char("badly formed label");
	YY_BREAK
	/****** LINE DIRECTIVES *******/
case 11:
YY_USER_ACTION
# line 179 "scan.y"

	YY_BREAK
case 12:
YY_USER_ACTION
# line 181 "scan.y"
{
									gLineNumber = atoi(yytext);
									BEGIN(Line2);
								}
	YY_BREAK
case 13:
YY_USER_ACTION
# line 186 "scan.y"
{
									yytext[yyleng - 1] = 0;
									gFileName = Name(gNames,
											NameID(gNames, yytext + 1));
								}
	YY_BREAK
case 14:
YY_USER_ACTION
# line 192 "scan.y"

	YY_BREAK
case 15:
YY_USER_ACTION
# line 194 "scan.y"
BEGIN(Newline);
	YY_BREAK
	/****** OPCODE MODE *******/
	/* Munch whitespace */
case 16:
YY_USER_ACTION
# line 200 "scan.y"

	YY_BREAK
	/* Instruction or macro name */
case 17:
YY_USER_ACTION
# line 204 "scan.y"
{
									int		k;

									for (k = 0; k < yyleng; k++)
										yytext[k] = toupper(yytext[k]);
									yytext[yyleng] = 0;
									yylval.id = NameID(gNames, yytext);
									BEGIN(Args);
									return(IDENT);
								}
	YY_BREAK
	/* Assignment */
case 18:
YY_USER_ACTION
# line 217 "scan.y"
{
									BEGIN(Args);
									return('=');
								}
	YY_BREAK
	/* Pseudo-opcodes */
case 19:
YY_USER_ACTION
# line 224 "scan.y"
Pseudo(ALIGN);
	YY_BREAK
case 20:
YY_USER_ACTION
# line 225 "scan.y"
Pseudo(ASCII);
	YY_BREAK
case 21:
YY_USER_ACTION
# line 226 "scan.y"
Pseudo(ASCIZ);
	YY_BREAK
case 22:
YY_USER_ACTION
# line 227 "scan.y"
Pseudo(ASSERT);
	YY_BREAK
case 23:
YY_USER_ACTION
# line 228 "scan.y"
Pseudo(BEGNN);
	YY_BREAK
case 24:
YY_USER_ACTION
# line 229 "scan.y"
Pseudo(BSS);
	YY_BREAK
case 25:
YY_USER_ACTION
# line 230 "scan.y"
Pseudo(BYTE);
	YY_BREAK
case 26:
YY_USER_ACTION
# line 231 "scan.y"
Pseudo(CODE);
	YY_BREAK
case 27:
YY_USER_ACTION
# line 232 "scan.y"
Pseudo(DATA);
	YY_BREAK
case 28:
YY_USER_ACTION
# line 233 "scan.y"
Pseudo(DBYT);
	YY_BREAK
case 29:
YY_USER_ACTION
# line 234 "scan.y"
Pseudo(DCI);
	YY_BREAK
case 30:
YY_USER_ACTION
# line 235 "scan.y"
Pseudo(SPACE);
	YY_BREAK
case 31:
YY_USER_ACTION
# line 236 "scan.y"
Pseudo(SPACE);
	YY_BREAK
case 32:
YY_USER_ACTION
# line 237 "scan.y"
Pseudo(END);
	YY_BREAK
case 33:
YY_USER_ACTION
# line 238 "scan.y"
Pseudo(ENDM);
	YY_BREAK
case 34:
YY_USER_ACTION
# line 239 "scan.y"
Pseudo('=');
	YY_BREAK
case 35:
YY_USER_ACTION
# line 240 "scan.y"
Pseudo(EXPAND);
	YY_BREAK
case 36:
YY_USER_ACTION
# line 241 "scan.y"
Pseudo(GLOBAL);
	YY_BREAK
case 37:
YY_USER_ACTION
# line 242 "scan.y"
Pseudo(IMPORT);
	YY_BREAK
case 38:
YY_USER_ACTION
# line 243 "scan.y"
Pseudo(IMPORT);
	YY_BREAK
case 39:
YY_USER_ACTION
# line 244 "scan.y"
Pseudo(MACRO);
	YY_BREAK
case 40:
YY_USER_ACTION
# line 245 "scan.y"
Pseudo(FLOAT);
	YY_BREAK
case 41:
YY_USER_ACTION
# line 246 "scan.y"
Pseudo(GLOBAL);
	YY_BREAK
case 42:
YY_USER_ACTION
# line 247 "scan.y"
Pseudo(HIBIT);
	YY_BREAK
case 43:
YY_USER_ACTION
# line 248 "scan.y"
Pseudo(LONG);
	YY_BREAK
case 44:
YY_USER_ACTION
# line 249 "scan.y"
Pseudo(LONG);
	YY_BREAK
case 45:
YY_USER_ACTION
# line 250 "scan.y"
Pseudo(WORD);
	YY_BREAK
case 46:
YY_USER_ACTION
# line 251 "scan.y"
Pseudo(ZERO);
	YY_BREAK
case 47:
YY_USER_ACTION
# line 252 "scan.y"
Pseudo(ZIMPORT);
	YY_BREAK
case 48:
YY_USER_ACTION
# line 253 "scan.y"
Pseudo(IF);
	YY_BREAK
case 49:
YY_USER_ACTION
# line 254 "scan.y"
Pseudo(ELSE);
	YY_BREAK
case 50:
YY_USER_ACTION
# line 255 "scan.y"
Pseudo(ENDIF);
	YY_BREAK
	/* Unknown pseudo-op */
case 51:
YY_USER_ACTION
# line 259 "scan.y"
{
									uerror("undefined pseudo-op");
									BEGIN(Ignore);
								}
	YY_BREAK
	/* Junk */
case 52:
YY_USER_ACTION
# line 266 "scan.y"
bad_char("bad opcode");
	YY_BREAK
	/****** ARGS MODE *******/
	/* Ignore whitespace once past the operand */
case 53:
YY_USER_ACTION
# line 272 "scan.y"

	YY_BREAK
	/* Integer constants	*/
case 54:
YY_USER_ACTION
# line 276 "scan.y"
{
									char	*s;

									for (yylval.intval = 0, s = yytext + 1;
											*s; s++)
										yylval.intval =
											(yylval.intval << 1) | (*s == '1');
									return(INTLIT);
								}
	YY_BREAK
case 55:
YY_USER_ACTION
# line 286 "scan.y"
{
									sscanf(yytext + 1, "%o", &yylval.intval);
									return(INTLIT);
								}
	YY_BREAK
case 56:
YY_USER_ACTION
# line 291 "scan.y"
{
									sscanf(yytext, "%d", &yylval.intval);
									return(INTLIT);
								}
	YY_BREAK
case 57:
YY_USER_ACTION
# line 296 "scan.y"
{
									sscanf(yytext + 2, "%x", &yylval.intval);
									return(INTLIT);
								}
	YY_BREAK
case 58:
YY_USER_ACTION
# line 301 "scan.y"
{
									sscanf(yytext + 1, "%x", &yylval.intval);
									return(INTLIT);
								}
	YY_BREAK
	/* Floating constants */
case 59:
# line 309 "scan.y"
case 60:
YY_USER_ACTION
# line 309 "scan.y"
{
									yylval.floatval = atof(yytext);
									return(FLOATLIT);
								}
	YY_BREAK
	/* Character constants	*/
case 61:
YY_USER_ACTION
# line 316 "scan.y"
{
									*(textptr = textbuf) = '\0';
									BEGIN(Squote);
								}
	YY_BREAK
	/* String constants	*/
case 62:
YY_USER_ACTION
# line 323 "scan.y"
{
									*(textptr = textbuf) = '\0';
									BEGIN(Dquote);
								}
	YY_BREAK
	/* Normal Label */
case 63:
YY_USER_ACTION
# line 330 "scan.y"
{
									yytext[yyleng] = 0;
									yylval.id = NameID(gNames, yytext);
									return(IDENT);
								}
	YY_BREAK
	/* Special symbols	*/
case 64:
YY_USER_ACTION
# line 338 "scan.y"
return(*yytext);
	YY_BREAK
case 65:
YY_USER_ACTION
# line 339 "scan.y"
return(*yytext);
	YY_BREAK
case 66:
YY_USER_ACTION
# line 340 "scan.y"
return(*yytext);
	YY_BREAK
case 67:
YY_USER_ACTION
# line 342 "scan.y"
return(LSB);
	YY_BREAK
case 68:
YY_USER_ACTION
# line 343 "scan.y"
return(LSB);
	YY_BREAK
case 69:
YY_USER_ACTION
# line 344 "scan.y"
return(MSB);
	YY_BREAK
case 70:
YY_USER_ACTION
# line 346 "scan.y"
return(LSH);
	YY_BREAK
case 71:
YY_USER_ACTION
# line 347 "scan.y"
return(RSH);
	YY_BREAK
case 72:
YY_USER_ACTION
# line 349 "scan.y"
return(LAND);
	YY_BREAK
case 73:
YY_USER_ACTION
# line 350 "scan.y"
return(LOR);
	YY_BREAK
case 74:
YY_USER_ACTION
# line 352 "scan.y"
return(EQU);
	YY_BREAK
case 75:
YY_USER_ACTION
# line 353 "scan.y"
return(NEQ);
	YY_BREAK
case 76:
YY_USER_ACTION
# line 354 "scan.y"
return(LE);
	YY_BREAK
case 77:
YY_USER_ACTION
# line 355 "scan.y"
return(GE);
	YY_BREAK
	/* Junk */
case 78:
YY_USER_ACTION
# line 359 "scan.y"
bad_char("bad operand");
	YY_BREAK
	/***** CHARACTER AND STRING CONSTANTS ******/
	/* Terminate string constant */
case 79:
YY_USER_ACTION
# line 365 "scan.y"
{
									yylval.textval.chars = GetMem(
										yylval.textval.len = textptr - textbuf);
									memcpy(yylval.textval.chars,
										textbuf, yylval.textval.len);
									BEGIN(Args);
									return(TEXTLIT);
								}
	YY_BREAK
	/* Terminate char constant */
case 80:
YY_USER_ACTION
# line 376 "scan.y"
{
									if (--textptr != textbuf)
										uerror("bad character constant");
									yylval.charval = *textptr;
									BEGIN(Args);
									return(CHARLIT);
								}
	YY_BREAK
	/* Special escaped chars */
case 81:
YY_USER_ACTION
# line 386 "scan.y"
add_char('\n', TRUE);
	YY_BREAK
case 82:
YY_USER_ACTION
# line 387 "scan.y"
add_char('\t', TRUE);
	YY_BREAK
case 83:
YY_USER_ACTION
# line 388 "scan.y"
add_char('\v', TRUE);
	YY_BREAK
case 84:
YY_USER_ACTION
# line 389 "scan.y"
add_char('\b', TRUE);
	YY_BREAK
case 85:
YY_USER_ACTION
# line 390 "scan.y"
add_char('\r', TRUE);
	YY_BREAK
case 86:
YY_USER_ACTION
# line 391 "scan.y"
add_char('\f', TRUE);
	YY_BREAK
case 87:
YY_USER_ACTION
# line 392 "scan.y"
add_char('\a', TRUE);
	YY_BREAK
case 88:
YY_USER_ACTION
# line 393 "scan.y"
add_char(0x1B, TRUE);
	YY_BREAK
case 89:
YY_USER_ACTION
# line 394 "scan.y"
add_char('\\', TRUE);
	YY_BREAK
case 90:
YY_USER_ACTION
# line 395 "scan.y"
add_char('\'', TRUE);
	YY_BREAK
case 91:
YY_USER_ACTION
# line 396 "scan.y"
add_char('"', TRUE);
	YY_BREAK
	/* End of line during quote? */
case 92:
# line 401 "scan.y"
case 93:
YY_USER_ACTION
# line 401 "scan.y"
{
									uerror("unterminated char or string constant");
									unput('\n');
									BEGIN(Ignore);
								}
	YY_BREAK
	/* Generic escaped chars -- just swallow backslash */
case 94:
YY_USER_ACTION
# line 409 "scan.y"

	YY_BREAK
	/* Octal character */
case 95:
YY_USER_ACTION
# line 413 "scan.y"
{
									int		result = 0;

									if (sscanf(yytext + 1, "%o", &result) != 1)
										cerror("can't scan octal character");
									if (result & ~0xFF)
										uerror("octal character out of bounds");
									add_char(result & 0xFF, FALSE);
								}
	YY_BREAK
	/* Hex character */
case 96:
YY_USER_ACTION
# line 425 "scan.y"
{
									int		result = 0;

									if (sscanf(yytext + 2, "%x", &result) != 1)
										cerror("can't scan hex character");
									if (result & ~0xFF)
										uerror("hex character out of bounds");
									add_char(result & 0xFF, FALSE);
								}
	YY_BREAK
case 97:
YY_USER_ACTION
# line 435 "scan.y"
{
									uerror("bad hex character constant");
								}
	YY_BREAK
case 98:
YY_USER_ACTION
# line 440 "scan.y"
{
									if (!isprint(*yytext))
										uerror("illegal character $%02X in char"
												" or string constant", *yytext);
									else
										add_char(*yytext, TRUE);
								}
	YY_BREAK
case YY_STATE_EOF(Squote):
case YY_STATE_EOF(Dquote):
# line 448 "scan.y"
{
									uerror("end of file in char or string constant");
									yyterminate();
								}
	YY_BREAK
	/* Comments extend to end of line */
case 99:
YY_USER_ACTION
# line 455 "scan.y"

	YY_BREAK
case 100:
YY_USER_ACTION
# line 457 "scan.y"
YY_FATAL_ERROR( "flex scanner jammed" );
	YY_BREAK
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = yy_cp - yytext_ptr - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
							yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr - 1; /* copy prev. char, too */
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a singled characater, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = yy_c_buf_p - yytext_ptr;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset = yy_c_buf_p - b->yy_ch_buf;

			b->yy_buf_size *= 2;
			b->yy_ch_buf = (char *)
				yy_flex_realloc( (void *) b->yy_ch_buf,
						 b->yy_buf_size );

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move - YY_MORE_ADJ == 1 )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	/* yytext begins at the second character in yy_ch_buf; the first
	 * character is the one which preceded it before reading in the latest
	 * buffer; it needs to be kept around in case it's a newline, so
	 * yy_get_previous_state() will have with '^' rules active.
	 */

	yytext_ptr = &yy_current_buffer->yy_ch_buf[1];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 240 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 240 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 239);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += dest - source;
		yy_bp += dest - source;
		yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	if ( yy_cp > yy_bp && yy_cp[-1] == '\n' )
		yy_cp[-2] = '\n';

	*--yy_cp = (char) c;


	/* Note: the formal parameter *must* be called "yy_bp" for this
	 * macro to now work correctly.
	 */
	YY_DO_BEFORE_ACTION; /* set up yytext again */
	}


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			yytext_ptr = yy_c_buf_p;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						{
						yy_c_buf_p =
						yytext_ptr + YY_MORE_ADJ;
						return EOF;
						}

					YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;
					break;

				case EOB_ACT_LAST_MATCH:
#ifdef __cplusplus
					YY_FATAL_ERROR(
					"unexpected last match in yyinput()" );
#else
					YY_FATAL_ERROR(
					"unexpected last match in input()" );
#endif
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );

	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );

	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	yy_flex_free( (void *) b->yy_ch_buf );
	yy_flex_free( (void *) b );
	}


#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif
	{
	b->yy_input_file = file;

	/* We put in the '\n' and start reading from [1] so that an
	 * initial match-at-newline will be true.
	 */

	b->yy_ch_buf[0] = '\n';
	b->yy_n_chars = 1;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[2] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[1];

	b->yy_is_interactive = file ? isatty( fileno(file) ) : 0;

	b->yy_fill_buffer = 1;

	b->yy_buffer_status = YY_BUFFER_NEW;
	}


#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		int new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}


static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}


static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}


#ifdef YY_USE_PROTOS
static void yy_fatal_error( const char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( 1 );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n - YY_MORE_ADJ; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, const char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
const char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( unsigned int size )
#else
static void *yy_flex_alloc( size )
unsigned int size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, unsigned int size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
unsigned int size;
#endif
	{
	return (void *) realloc( ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}
# line 457 "scan.y"


static	int
LoadBuf(char *buf, int max)
{
	int			space, xfer, num;
	static int	posn = 0;

/* Read up to next newline (or max chars) */

	if (fgets(buf, max, yyin) == NULL)
		return(YY_NULL);
	num = strlen(buf);

/* Transfer into line buffer for listing */

	if ((space = MAX_LISTING_LINE - posn - 1) > 0)
	{
		xfer = buf[num - 1] == '\n' ? num - 1 : num;
		xfer = (space < xfer) ? space : xfer;
		memcpy(gLineBuf + posn, buf, xfer);
		posn += xfer;
	}

/* If we got a newline, terminate and reset line buffer */

	if (buf[num - 1] == '\n')
	{
		gLineBuf[posn] = 0;
		posn = 0;
		if (gDebug)
			fprintf(stderr, "READ--->%s\n", gLineBuf);
	}

/* Return number read into buffer */

	return(num);
}

