
/*
 * names.h
 *
 * (C) Copyright 1995 Archie L. Cobbs
 */

/*
 * DEFINITIONS
 */

	OpaqueDecl(Names);

/*
 * FUNCTIONS
 */

	extern	Names			NewNames(unsigned capacity);
	extern	unsigned		NameID(Names db, char *name);
	extern	char			*Name(Names db, unsigned id);
	extern	Names			NamesLoad(FILE *fp, unsigned size);
	extern	unsigned		NamesDump(FILE *fp, Names db);

