
;
; This is here just so "foo.s" has something to link with
;

		.import	bignum

		.export	extern_zp
		.export	extern_1
		.export	extern_2
		.export	extern_undef

extern_zp	=	$06

		.code
extern_1	lda	extern_zp
		jsr	label1
		rts

		.data
extern_2	.long	globdat

		.bss
extern_undef	.ds	1

never_heard_of_that_one	.ds	1
		.export	never_heard_of_that_one

