
/*
 * util.h
 *
 * (C) Copyright 1995 Archie L. Cobbs
 */

/*
 * DEFINITIONS
 */

/* Generic stuff */

	typedef	unsigned char		Byte;

	#ifndef	NULL
	#define	NULL				0
	#endif

	#define	TRUE				1
	#define	FALSE				0

	#define	OpaqueDecl(type)	struct type; typedef struct type *type;
	#define	OpaqueDefn(type)	struct type
	#define	NewOpaque(type)		((type) GetMem((sizeof(struct type))))

/*
 * GLOBAL VARIABLES
 */

	extern	char			*gProgName;
	extern	char			*gUsageMsg;

/*
 * FUNCTIONS
 */

/* Memory allocation */

	extern	void			*GetMem(int len);
	extern	void			*GetMoreMem(void *mem, int len);

/* Unusual exits */

	extern	volatile void	Panic(char *msg, ...);
	extern	volatile void	Usage(char *msg, ...);


